﻿using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Microsoft.CSharp;

namespace Articles.ScriptingCSharp.ScriptFramework.CSharp
{
    public class CSharpScriptCompiler
    {
        private CodeDomProvider m_codeCompiler;

        /// <summary>
        /// Initializes a new instance of the <see cref="CSharpScriptCompiler" /> class.
        /// </summary>
        public CSharpScriptCompiler()
        {
            Dictionary<string, string> compilerParameters = new Dictionary<string,string>();
            compilerParameters["CompilerVersion"] = "v4.0";
            m_codeCompiler = CSharpCodeProvider.CreateProvider(
                "cs",
                compilerParameters);
        }

        /// <summary>
        /// Compiles given source file.
        /// </summary>
        /// <param name="m_currentFile">The source file to be compiled.</param>
        public Action Compile(CSharpScriptFile m_currentFile)
        {
            //Set compiler parameters
            CompilerParameters compilerParameters = new CompilerParameters();
            compilerParameters.GenerateExecutable = false;
            compilerParameters.GenerateInMemory = true;
            compilerParameters.OutputAssembly = "ScriptedAssembly";
            compilerParameters.ReferencedAssemblies.AddRange(
                m_currentFile.ReferencedAssemblies.ToArray());

            //Compile the code and return the result or throw an exception in case of an error
            CompilerResults results = m_codeCompiler.CompileAssemblyFromSource(
                compilerParameters, 
                m_currentFile.ScriptContent);
            if (results.Errors.Count > 0)
            {
                StringBuilder compilerErrors = new StringBuilder();
                foreach (var actError in results.Errors)
                {
                    compilerErrors.Append("" + actError + Environment.NewLine);
                }
                throw new InvalidOperationException(
                    "Unable to compile scripts: " + 
                    compilerErrors.ToString());
            }
            else if (results.CompiledAssembly == null)
            {
                throw new InvalidOperationException(
                    "Unable to compile scripts: " + 
                    "No result assembly from compiler!");
            }

            //Try to get main class
            Assembly compiledAssembly = results.CompiledAssembly;
            Type scriptClass = compiledAssembly.GetType("ScriptedClass");
            if (scriptClass == null) 
            { 
                throw new InvalidOperationException(
                    "Unable to compile scripts: " +
                    "class ScriptedClass not found!");
            }

            //Try to get main method
            MethodInfo scriptMethod = scriptClass.GetMethod(
                "Execute", 
                BindingFlags.Static | BindingFlags.Public);
            if (scriptMethod == null) 
            {
                throw new InvalidOperationException(
                    "Unable to compile scripts: " +
                    "method Execute not found within class ScriptedClass!"); }


            return () => scriptMethod.Invoke(null, null);
        }
    }
}
