﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Articles.ScriptingCSharp.ScriptFramework.CSharp;
using ICSharpCode.SharpDevelop.Dom;

namespace Articles.ScriptingCSharp.ScriptableApplication
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Lazy<CSharpScriptCompiler> m_scriptCompiler;   
        private CSharpScriptFile m_currentFile;

        /// <summary>
        /// Initializes a new instance of the <see cref="MainWindow" /> class.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            m_currentFile = new CSharpScriptFile();
            this.DataContext = m_currentFile;
            //this.TextEditor.Text = m_currentFile.ScriptContent;

            m_scriptCompiler = new Lazy<CSharpScriptCompiler>();
        }

        /// <summary>
        /// Called when user wants to compile the script.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        private void OnMenuCompileClick(object sender, RoutedEventArgs e)
        {
            //m_currentFile.ScriptContent = this.TextEditor.Text;

            //Compile and execute current script
            Action compiledMethod = m_scriptCompiler.Value.Compile(m_currentFile);
            compiledMethod();
        }

        /// <summary>
        /// Called when user wants to add an assembly.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        private void OnMenuAssembliesAddClick(object sender, RoutedEventArgs e)
        {
            AddAssemblyDialog dlgAddAssembly = new AddAssemblyDialog();
            if (dlgAddAssembly.ShowDialog().Value)
            {
                if (!m_currentFile.ReferencedAssemblies.Contains(dlgAddAssembly.AssemblyName))
                {
                    m_currentFile.ReferencedAssemblies.Add(dlgAddAssembly.AssemblyName);
                }
            }
        }

        /// <summary>
        /// Called when user wants to remove and assembly.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="RoutedEventArgs" /> instance containing the event data.</param>
        private void OnMenuAssembliesRemoveClick(object sender, RoutedEventArgs e)
        {
            string selectedAssembly = LstAssemblies.SelectedItem as string;
            if (selectedAssembly != null)
            {
                m_currentFile.ReferencedAssemblies.Remove(selectedAssembly);
            }
        }
    }
}
